# -*- coding: utf-8 -*-
##------------------------------------------
## OCR Test with easyocr
##
##               2022.05.09 Masahiro Izutsu
##------------------------------------------
## easytest4.py

# Color Escape Code
GREEN = '\033[1;32m'
RED = '\033[1;31m'
NOCOLOR = '\033[0m'
YELLOW = '\033[1;33m'

# 定数定義
LINE_WORD_BOX_COLOR = (0, 0, 240)
WORD_BOX_COLOR = (255, 0, 0)
CONTENTS_COLOR = (0, 180, 0)
from os.path import expanduser
DEF_INPUT_FILE = expanduser('images/japanese.jpg')

# import処理
from PIL import Image
import sys

import easyocr
import cv2
import argparse
import myfunction
import numpy as np
import mylib_gui
import platform

# タイトル・バージョン情報
title = 'OCR Test with easyocr'
print(GREEN)
print('--- {} ---'.format(title))
print(' OpenCV version {} '.format(cv2.__version__))
print(NOCOLOR)

# Parses arguments for the application
def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--image', metavar = 'IMAGE_FILE', type = str, default = DEF_INPUT_FILE,
            help = 'Absolute path to image file. Default value is \'' + DEF_INPUT_FILE + '\'')
    parser.add_argument('-p', '--prosess', metavar = 'PROCESS',
            default = 'n',
            help = 'Preprocessing flag.(y/n) Default value is \'n\'')
    parser.add_argument('-d', '--linedel', metavar = 'LINEDEL',
            default = 'n',
            help = 'Line delete flag.(y/b/n) Default value is \'n\'')
    parser.add_argument('--maxsize', metavar = 'MAXSIZE',
            default = 0,
            help = 'Image max size (free=0). Default value is 0')
    parser.add_argument('--log', metavar = 'LOG',
            default = 'n',
            help = 'Log output flag.(y/n) Default value is \'n\'')
    parser.add_argument('-t', '--title', metavar = 'TITLE',
            default = 'y',
            help = 'Program title flag.(y/n) Default value is \'y\'')
    parser.add_argument('-o', '--out', metavar = 'IMAGE_OUT',
            default = 'non',
            help = 'Processed image file path. Default value is \'non\'')
    return parser

# モデル基本情報の表示
def display_info(image, prosess, linedel, maxsize, log, titleflg, outpath):
    print(YELLOW + title + ': Starting application...' + NOCOLOR)
    print('   - ' + YELLOW + 'Image File   : ' + NOCOLOR, image)
    print('   - ' + YELLOW + 'Preprocessing: ' + NOCOLOR, prosess)
    print('   - ' + YELLOW + 'Line delete  : ' + NOCOLOR, linedel)
    print('   - ' + YELLOW + 'Max size     : ' + NOCOLOR, maxsize)
    print('   - ' + YELLOW + 'Log frag     : ' + NOCOLOR, log)
    print('   - ' + YELLOW + 'Program Title: ' + NOCOLOR, titleflg)
    print('   - ' + YELLOW + 'Processed out: ' + NOCOLOR, outpath)

# 画像の前処理
def img_preproces(img):
    # グレイスケール演算
    im_gray = 0.299 * img[:,:,2] + 0.587 * img[:,:,1] + 0.114 * img[:,:,0]
    im_gray8 = np.uint8(im_gray)
    # 大津アルゴリズムでは thresh, maxvalは無視されてしきい値は自動で設定される
    ret, im_gray8 = cv2.threshold(im_gray8, thresh=0, maxval=255, type=cv2.THRESH_BINARY + cv2.THRESH_OTSU)
    # すべてのチャンネル
    img[:,:,0] = im_gray8
    img[:,:,1] = im_gray8
    img[:,:,2] = im_gray8
    return img

# 罫線消去
def delete_line(img):
    # 自動パラメータの計算
    h, w = img.shape[:2]
    thr = 100
    lln = int(w/18)
    if lln < 44:
        lln = 44
    gap = int(w/1000) + 4
    print('\nThreshhold={}, MinLineLength={}, MaxLineGap={},  width={}, height={}'.format(thr, lln, gap, w, h))

    imgw = img.copy()
    # グレースケール
    gray = cv2.cvtColor(imgw, cv2.COLOR_BGR2GRAY)

    # 2値化
    ret, gray = cv2.threshold(gray, thresh=0, maxval=255, type=cv2.THRESH_BINARY + cv2.THRESH_OTSU)

    ## 反転 ネガポジ変換
    gray = cv2.bitwise_not(gray)
    lines = cv2.HoughLinesP(gray, rho=1, theta=np.pi/360, threshold=thr, minLineLength=lln, maxLineGap=gap)

    if lines is not None:
        for line in lines:
            x1, y1, x2, y2 = line[0]

            # 線を消す(白で線を引く)
            imgw = cv2.line(imgw, (x1,y1), (x2,y2), (255,255,255), 3)
    return imgw

# ** main関数 **
def main():
    # Argument parsing and parameter setting
    ARGS = parse_args().parse_args()
    input_stream = ARGS.image
    prosess = ARGS.prosess
    linedel = ARGS.linedel
    maxsize = int(ARGS.maxsize)
    logflg = ARGS.log
    titleflg = ARGS.title
    outpath = ARGS.out

    # 情報表示
    display_info(input_stream, prosess, linedel, maxsize, logflg, titleflg, outpath)

    # OpenCV でイメージを読む
    frame = cv2.imread(input_stream)
    if frame is None:
        print(RED + "\nUnable to read the input." + NOCOLOR)
        quit()

    if maxsize > 300:
        # アスペクト比を固定してリサイズ
        img_h, img_w = frame.shape[:2]
        if (img_w > img_h):
            if (img_w > maxsize):
                height = round(img_h * (maxsize / img_w))
                frame = cv2.resize(frame, dsize = (maxsize, height))
        else:
            if (img_h > maxsize):
                width =  round(img_w * (maxsize / img_h))
                frame = cv2.resize(frame, dsize = (width, maxsize))

    # メッセージ作成
    h, w = frame.shape[:2]
    st_pram = 'pros={}, linedel={}, width={}, height={}'.format(prosess, linedel, w, h)

    # fontスケール(仮設定)
    img_h, img_w = frame.shape[:2]
    font_scale = 20
    if img_w > 2000:
        font_scale = 40

    # 画像の前処理
    if (prosess == 'y'):        # モノクロ・2値化 フォアグラウンド処理
        frame = img_preproces(frame)

    if (linedel == 'y'):         # 罫線除去 フォアグラウンド処理
        frame = delete_line(frame)
        frame_pl = cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)
    elif (linedel == 'b'):   # 罫線除去 バックグラウンド処理
        frame_pl = delete_line(frame)
        frame_pl = cv2.cvtColor(frame_pl, cv2.COLOR_RGB2BGR)
    else:                       # 罫線除去なし
        frame_pl = cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)

    # 日本語フォント指定
    if platform.system()=='Windows':
       fontPIL = 'meiryo.ttc'                  # ゴシック体
    else:
       fontPIL = 'NotoSansCJK-Bold.ttc'         # ゴシック体

    # 文字認識処理
    reader = easyocr.Reader(['ja', 'en'], gpu=False)
    result = reader.readtext(frame)

    # 取得データ
    print('\n---------------------------')
    for obj in result:
        print(obj[1])

    for obj in result:
        xmin = int(obj[0][0][0])                # 結果が浮動小数点になる場合があるので
        ymin = int(obj[0][0][1])
        xmax = int(obj[0][2][0])
        ymax = int(obj[0][2][1])
        text = obj[1]
        confidence = obj[2]

        cv2.rectangle(frame, (xmin, ymin), (xmax, ymax), color=LINE_WORD_BOX_COLOR, thickness=3)
        st_score = '#Score {:.3f}:  '.format(confidence) + text
        myfunction.cv2_putText(img = frame,
                               text = st_score,
                               org = (xmin, ymin - 4),
                               fontFace = fontPIL,
                               fontScale = font_scale,
                               color = CONTENTS_COLOR,
                               mode = 0)

        if (logflg == 'y'):
            print('\ncontents: ', text)
            print('position: ', xmin, ymin, '-', xmax, ymax)
            print('confidence: ', confidence)

    print('---------------------------\n')

    # タイトル描画
    if (titleflg == 'y'):
        cv2.putText(frame, title, (10, 30), cv2.FONT_HERSHEY_DUPLEX, fontScale=0.8, color=(200, 200, 0), lineType=cv2.LINE_AA)
        cv2.putText(frame, st_pram, (50, 50), cv2.FONT_HERSHEY_DUPLEX, fontScale=0.5, color=(0, 0, 0), lineType=cv2.LINE_AA)

    # 画像表示 
    cv2.namedWindow(title, flags=cv2.WINDOW_AUTOSIZE | cv2.WINDOW_GUI_NORMAL) 
    cv2.imshow(title, frame)

    # 処理結果の記録(静止画)
    if (outpath != 'non'):
        cv2.imwrite(outpath, frame)

    # 何らかのキーが押されたら終了 
    while(True):
        key = cv2.waitKey(1)
        if key == 27 or key == 113:                          # 'esc' or 'q'
            break
        if not mylib_gui._is_visible(title):           # 'Close' button
            break

    cv2.destroyAllWindows()

# main関数エントリーポイント(実行開始)
if __name__ == "__main__":
    sys.exit(main())

