# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library File treatment  Ver 0.04
##
##               2022.01.27 Masahiro Izutsu
##------------------------------------------
## mylib_file.py
##  2022/03/09  ver 0.03    update
##  2022/04/13  Ver 0.04    removed from class 'Document' after v1.19

import os, sys
import fitz
import glob
import cv2
from pathlib import Path
import shutil

class FileTreatment:
    logf = False
    
    # 初期化
    #   flg:        ログ出力フラグ
    def __init__(self, flg = False):
        self.logf = flg

    # PDFファイルから画像ファイルを抽出する
    #   XXXXXXXX.pdf → XXXXXXXX/XXXXXXXX_ページ数_番号.イメージ拡張子
    def pdf2img(self, fname):
        dstdir = os.path.splitext(fname)[0]
        dstn = os.path.basename(dstdir)
        topn = Path(dstdir).stem
        os.makedirs(dstdir, exist_ok=True)
        with fitz.open(fname) as doc:
            for i, page in enumerate(doc):
                for j, img in enumerate(page.get_images()):
                    x = doc.extract_image(img[0])
                    name = os.path.join(dstdir, f"{topn:s}_{i:04}_{j:02}.{x['ext']}")
                    with open(name, "wb") as ofh:
                        ofh.write(x['image'])
                        if self.logf:
                            fsname = os.path.basename(name)
                            print(' Image file: ', dstn + '/' + fsname)
        return dstdir

    # ファイルパスからファイル一覧を得る
    def get_file_list(self, dirpass):
        result = sorted(glob.glob(dirpass))
        if self.logf:
            print(' File list: ', result)
        return result

    # ファイルパスから拡張子で選択したファイル一覧を得る
    def get_file_list_sel(self, dirpass, sel_ext):
        flist = sorted(glob.glob(dirpass))
        result = []
        for name in flist:
            for ext in sel_ext:
                if ext == os.path.splitext(name)[1]:
                    result.append(name)

        if self.logf:
            print(' Select list: ', result)
        return result

    # ファイルを移動する XXXX/XXXXX.xxx ←→  XXXX-err/XXXXX.xxx
    def move_file(self, filename):
        fname = os.path.basename(filename)
        fpath = os.path.dirname(filename)
        if fpath[-4:] == '-err':
            tgtpath = fpath[0:-4]
            if not os.path.exists(tgtpath):
                os.mkdir(tgtpath)
            tgt = tgtpath + '/' + fname
        else:
            tgtpath = fpath + '-err'
            if not os.path.exists(tgtpath):
                os.mkdir(tgtpath)
            tgt = tgtpath + '/' + fname
        shutil.move(filename,tgt)
        if self.logf:
            print(' Move file {} → {}'.format(filename,tgt))

    # -err フォルダのチェック
    def ckeck_erfolder(self, filename):
        fname = os.path.basename(filename)
        fpath = os.path.dirname(filename)
        errf = True if fpath[-4:] == '-err' else False
        return errf

    # ファイル末尾のチェック
    def ckeck_filetail(self, filename, ftail):
        fname = os.path.basename(filename)
        f_base = os.path.splitext(fname)[0]
        n = len(ftail)
        f = True if f_base[-n:] == ftail else False
        return f

    # ファイルを保存する XXXXX.xxx ←→  'dname'/XXXXX.xxx
    def save_file(self, filename, dname = 'org'):
        fname = os.path.basename(filename)
        fpath = os.path.dirname(filename)
        tgtpath = fpath + '/' + dname
        if not os.path.exists(tgtpath):
            os.mkdir(tgtpath)
        tgt = tgtpath + '/' + fname
        shutil.move(filename,tgt)
        if self.logf:
            print(' Save file {} → {}'.format(filename,tgt))

    # イメージファイルを時計回りに90°回転
    def img_rotate90(self, filelist):
        for fname in filelist:
            self.img_rotate90_file(fname)

    def img_rotate90_file(self, filename):
        img = cv2.imread(filename)
        img_rotate_90_clockwise = cv2.rotate(img, cv2.ROTATE_90_CLOCKWISE)
        cv2.imwrite(filename, img_rotate_90_clockwise)
        if self.logf:
            fsname = os.path.basename(filename)
            print(' Image rotate_90: ', fsname)

    # イメージファイルを180°回転
    def img_rotate180(self, filelist):
        for fname in filelist:
            self.img_rotate180_file(fname)

    def img_rotate180_file(self, filename):
        img = cv2.imread(filename)
        img_rotate_180 = cv2.rotate(img, cv2.ROTATE_180)
        cv2.imwrite(filename, img_rotate_180)
        if self.logf:
            fsname = os.path.basename(filename)
            print(' Image rotate_180: ', fsname)

    # イメージファイルを反時計回りに270°回転
    def img_rotate270(self, filelist):
        for fname in filelist:
            self.img_rotate270_file(fname)

    def img_rotate270_file(self, filename):
        img = cv2.imread(filename)
        img_rotate_90_counterclockwise = cv2.rotate(img, cv2.ROTATE_90_COUNTERCLOCKWISE)
        cv2.imwrite(filename, img_rotate_90_counterclockwise)
        if self.logf:
            fsname = os.path.basename(filename)
            print(' Image rotate_270: ', fsname)


##---------------
def main():
    my_file = FileTreatment(True)

    '''
    # pdf >> image
    pdflist = my_file.get_file_list('data/*.pdf')
    for name in pdflist:
        dist = my_file.pdf2img(name)
        if name == 'data/20190920095901.pdf':
            lst = my_file.get_file_list(dist + '/*')
            my_file.img_rotate270(lst)
    '''

    from tkinter import filedialog

    # PDFファイルからイメージファイルを抽出
    dirname = filedialog.askdirectory(initialdir = './', title = '対象ディレクトリを選択', mustexist = True)
    if len(dirname) > 0:
        pdflist = my_file.get_file_list(dirname + '/*.pdf')
        for name in pdflist:
            dist = my_file.pdf2img(name)

    '''
    # イメージファイルを抽出
    dirname = filedialog.askdirectory(initialdir = './', title = '対象ディレクトリを選択', mustexist = True)
    if len(dirname) > 0:
        imglist = my_file.get_file_list(dirname + '/*')
        sel_ext = {'.bmp', '.png', '.jpeg', '.jpg', '.tif'}
        sellist = my_file.get_file_list_sel(dirname + '/*', sel_ext)
    '''

    '''
    # イメージファイルを回転
    dirname = filedialog.askdirectory(initialdir = './', title = '対象ディレクトリを選択', mustexist = True)
    if len(dirname) > 0:
        imglist = my_file.get_file_list(dirname + '/*')
        my_file.img_rotate270(imglist)
    '''

    '''
    # ファイルを移動する XXXX/XXXXX.xxx ←→  XXXX-err/XXXXX.xxx
    filename = filedialog.askopenfilename(title = "画像ファイルを開く",
            filetypes = [("Image file", ".bmp .png .jpg .jpeg .tif"), ("Bitmap", ".bmp"), ("PNG", ".png"), ("JPEG", ".jpg"), ("JPEG", ".jpeg"), ("Tiff", ".tif") ],
            initialdir = "./"
    )
    if len(filename) > 0:
        ret = my_file.move_file(filename)
        print(my_file.ckeck_erfolder(filename))
    '''

    '''
    # ファイルを保存する XXXXX.xxx ←→  'dname'/XXXXX.xxx
    filename = filedialog.askopenfilename(title = "画像ファイルを開く",
            filetypes = [("Image file", ".bmp .png .jpg .jpeg .tif"), ("Bitmap", ".bmp"), ("PNG", ".png"), ("JPEG", ".jpg"), ("JPEG", ".jpeg"), ("Tiff", ".tif") ],
            initialdir = "./"
    )
    if len(filename) > 0:
        my_file.save_file(filename, 'tilt')
    '''

    '''
    # ファイルの末尾を調べる
    filename = filedialog.askopenfilename(title = "画像ファイルを開く",
            filetypes = [("Image file", ".bmp .png .jpg .jpeg .tif"), ("Bitmap", ".bmp"), ("PNG", ".png"), ("JPEG", ".jpg"), ("JPEG", ".jpeg"), ("Tiff", ".tif") ],
            initialdir = "./"
    )
    if len(filename) > 0:
        print(my_file.ckeck_filetail(filename, '_tilt'))
    '''

if __name__ == "__main__":
    main()

