# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library image frame with OpenCV
##
##               2021.12.20 Masahiro Izutsu
##------------------------------------------
## mylib_frame.py

import cv2
import numpy as np

class ImageFrame:
    max_min = 300       # 画像をリサイズする最小ピクセル数
    frame_org = None    # 元画像イメージ
    frame_res = None    # リサイズされた画像イメージ
    org_h = 0           # 元画像の高さ
    org_w = 0           # 　　　　幅
    res_h = 0           # リサイズ画像の高さ
    res_w = 0           # 　　　　　　　幅
    scale_h = 0.0       # リサイズ画像の比率 高さ
    scale_w = 0.0       # 　　　　　　　　　 幅
    screen_h = 0        # 表示ウインドウの最大高さ
    screen_w = 0        # 　　　　　　　　　　幅
    
    def __init__(self, img, max_min = 300):
        self.frame_org = img
        self.max_min = max_min
        self.org_h, self.org_w = img.shape[:2]
    
    def set_screen_size(self, height, width):
        self.screen_h = height
        self.screen_w = width

    # アスペクト比を固定して画像をリサイズ
    #  boxl: 大きさ (-1=画面最大, 0=そのまま, max_min> boxlに収まるサイズ
    #  zoomf: 指定より小さい画像の拡大フラグ
    def frame_resize(self, boxl = 0, zoomf = False):
        if self.frame_org is not None:
            if boxl == -1:
                maxsize = self.screen_h if self.screen_h < self.screen_w else self.screen_w
            else:
                maxsize = boxl

            if maxsize > self.max_min:
                if self.org_w > self.org_h: # 横長(landscape)
                    if self.org_w > maxsize:
                        self.res_h = round(self.org_h * (maxsize / self.org_w))
                        self.res_w = maxsize
                    elif zoomf:             #  指定より小さい画像を拡大する
                        self.res_h = round(self.org_h * (maxsize / self.org_w))
                        self.res_w = maxsize
                    else:                   #  指定より小さい画像のまま                        
                        self.res_h = self.org_h
                        self.res_w = self.org_w
                else:                       # 縦長(portrait)
                    if self.org_h > maxsize:
                        self.res_h = maxsize
                        self.res_w =  round(self.org_w * (maxsize / self.org_h))
                    elif zoomf:             #  指定より小さい画像を拡大する
                        self.res_h = maxsize
                        self.res_w =  round(self.org_w * (maxsize / self.org_h))
                    else:                   #  指定より小さい画像のまま                        
                        self.res_h = self.org_h
                        self.res_w = self.org_w
                
                self.frame_res = cv2.resize(self.frame_org, dsize = (self.res_w, self.res_h))
            
            else:
                self.frame_res = self.frame_org.copy()
            
            self.res_h, self.res_w = self.frame_res.shape[:2]
            self.scale_h = self.res_h / self.org_h
            self.scale_w = self.res_w / self.org_w

        return self.frame_res

    def get_screen_size(self):
        return self.screen_h, self.screen_w

    def get_scale(self):
        return self.scale_h, self.scale_w
    
    def get_original_size(self):
        return self.org_h, self.org_w
    
    def get_resize_size(self):
        return self.res_h, self.res_w
    
    def get_res2org_xy(self, rx, ry):
        if self.scale_h > 0 and self.scale_w > 0:
            ox = round(rx / self.scale_w)
            oy = round(ry / self.scale_h)
        else:
            ox = 0
            oy = 0
        return ox, oy
    
    def get_org2res_xy(self, ox, oy):
        if self.scale_h > 0 and self.scale_w > 0:
            rx = round(ox * self.scale_w)
            ry = round(oy * self.scale_h)
        else:
            rx = 0
            ry = 0
        return rx, ry

# テストルーチン
if __name__ == "__main__":
    window_name = 'ImageFrame class'
#    image = np.zeros((2000, 4000, 3), np.uint8)
#    image = np.zeros((2000, 1200, 3), np.uint8)
    image = np.zeros((1500, 1000, 3), np.uint8)
#    image = np.zeros((1000, 1500, 3), np.uint8)
#    image = np.zeros((500, 1000, 3), np.uint8)
#    image = np.zeros((1000, 500, 3), np.uint8)
    imgfr = ImageFrame(image)               # 初期化
    imgfr.set_screen_size(1680, 1050)

    image_res = imgfr.frame_resize(-1, True)
    cv2.namedWindow(window_name, flags=cv2.WINDOW_AUTOSIZE | cv2.WINDOW_GUI_NORMAL) 
    cv2.imshow(window_name, image_res)
    
    screen_h, screen_w = imgfr.get_screen_size()
    print('screen_h; ', screen_h, '  screen_w: ', screen_w)    
    
    scale_h, scale_w = imgfr.get_scale()
    print('scale_h; ', scale_h, '  scale_w: ', scale_w)    
    
    org_h, org_w = imgfr.get_original_size()
    print('org_h; ', org_h, '  org_w: ', org_w)    
    
    res_h, res_w = imgfr.get_resize_size()
    print('res_h; ', res_h, '  res_w: ', res_w)    

    ox, oy = imgfr.get_res2org_xy(400, 500)
    print('ox: ', ox, '  oy: ', oy)    

    rx, ry = imgfr.get_org2res_xy(500, 500)
    print('rx: ', rx, '  ry: ', ry)    
    
    while(True):
        key = cv2.waitKey(1)
        if key == 27 or key == 113:         # 'esc' or 'q'
            break

    cv2.destroyAllWindows()

