# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library image preprocessing with OpenCV
##
##               2021.12.20 Masahiro Izutsu
##------------------------------------------
## mylib_preprocess.py

import cv2
import numpy as np
import math

class ImagePreprocess:
    logf = False
    
    def __init__(self, flg = False):
        self.logf = flg
    
    ## カラー原画 → グレイスケール → 白黒 2値画像  thresh:しきい値(0=大津アルゴリズム)
    def img_binarization(self, img, thresh = 0):
        type = cv2.THRESH_BINARY + cv2.THRESH_OTSU if thresh == 0 else cv2.THRESH_BINARY
        imgw = img.copy()
        # グレイスケール演算
        im_gray = 0.299 * img[:,:,2] + 0.587 * img[:,:,1] + 0.114 * img[:,:,0]
        im_gray8 = np.uint8(im_gray)
        # 大津アルゴリズムでは thresh, maxvalは無視されてしきい値は自動で設定される
#        ret, im_gray8 = cv2.threshold(im_gray8, thresh=0, maxval=255, type=cv2.THRESH_BINARY + cv2.THRESH_OTSU)
        ret, im_gray8 = cv2.threshold(im_gray8, thresh=thresh, maxval=255, type=type)
        # すべてのチャンネル
        imgw[:,:,0] = im_gray8
        imgw[:,:,1] = im_gray8
        imgw[:,:,2] = im_gray8
        return imgw

    ## 罫線消去
    def delete_line(self, img):
        imgw = img.copy()
        # グレースケール
        gray = cv2.cvtColor(imgw, cv2.COLOR_BGR2GRAY)
        # 2値化
        ret, gray = cv2.threshold(gray, thresh=0, maxval=255, type=cv2.THRESH_BINARY + cv2.THRESH_OTSU)
        ## 反転 ネガポジ変換
        gray = cv2.bitwise_not(gray)
        # ハフライン検出自動パラメータの計算
        thr, lln, gap = self.get_autohoughLine(img)
        lines = cv2.HoughLinesP(gray, rho=1, theta=np.pi/360, threshold=thr, minLineLength=lln, maxLineGap=gap)

        if lines is not None:
            for line in lines:
                x1, y1, x2, y2 = line[0]

                # 線を消す(白で線を引く)
                imgw = cv2.line(imgw, (x1,y1), (x2,y2), (255,255,255), 3)
        return imgw

    ## 画像の回転(OpenCV)
    def rotate_img(self, img, angle, boder = (255, 255, 255)):
        # 画像サイズ(横, 縦)から中心座標を求める
        size = tuple([img.shape[1], img.shape[0]])
        center = tuple([size[0] // 2, size[1] // 2])
        # 回転の変換行列を求める(画像の中心, 回転角度, 拡大率)
        mat = cv2.getRotationMatrix2D(center, angle, scale=1.0)
        # アフィン変換(画像, 変換行列, 出力サイズ, 補完アルゴリズム)
        rot_img = cv2.warpAffine(img, mat, size, flags=cv2.INTER_CUBIC, borderMode=cv2.BORDER_CONSTANT, borderValue=boder)
        return rot_img


    # 色の検出 (HSV)
    #  入力: mode 変換モード(0= 指定領域, 1= 指定外領域, 2= 指定領域を白に
    #        hmin,hmax H領域 (0 - 359°)
    #        smin,smax S領域 (0 - 100%)
    #        vmin,vmax V領域 (0 - 100%)
    def detect_color(self, img, hmin, hmax, smin, smax, vmin, vmax, mode):
        hmin1, hmax1, hmin2, hmax2 = self.calc_hue_area(hmin, hmax)
        # 0-100% >> 0-255
        smin = int(round(smin * 255 / 100))
        smax = int(round(smax * 255 / 100))
        vmin = int(round(vmin * 255 / 100))
        vmax = int(round(vmax * 255 / 100))
        if self.logf:
            print('\n Input range: min= {}, max= {}'.format(hmin, hmax))
            print(' H area: min1= {}, max1= {}, min2= {}, max2= {}'.format(hmin1, hmax1, hmin2, hmax2))
            print(' S area: min= {}, max= {}'.format(smin, smax))
            print(' V area: min= {}, max= {}'.format(vmin, vmax))

        # HSV色空間に変換
        hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)

        # HSVの値域 1
        hsv_min = np.array([hmin1,smin,vmin])
        hsv_max = np.array([hmax1,smax,vmax])
        mask = cv2.inRange(hsv, hsv_min, hsv_max)

        # HSVの値域 2
        if hmin2 < 180 and hmax2 < 180:
            hsv_min = np.array([hmin2,smin,vmin])
            hsv_max = np.array([hmax2,smax,vmax])
            mask2 = cv2.inRange(hsv, hsv_min, hsv_max)
            mask = mask + mask2

        # mask 領域のマスク (255：指定、0：指定以外)
        # mask_inv 領域のマスク反転 (0：指定、255：指定以外)
        mask_inv = cv2.bitwise_not(mask)

        if mode == 0:
            msk = mask
        else:
            msk = mask_inv

        # マスキング処理
        masked_img = cv2.bitwise_and(img, img, mask=msk)

        if mode == 2:
            # 特定の色を別の色に置換する
            before_color = [0, 0, 0]
            after_color = [255, 255, 255]
            masked_img[np.where((masked_img == before_color).all(axis=2))] = after_color

        return mask, masked_img

    ## イメージ処理の実行
    ##  入力: process_mode
    ##          1= カラー原画 → グレイスケール → 白黒 2値画像
    ##          2= 罫線消去
    ##          3= 画像の傾き補正
    ##          4= 印影除去
    ##          5= 黒抽出
    def image_processing_execution(self, img, process_mode):
        img1 = None
        if process_mode == 0:
            # no operation
            img1 = img
        if process_mode == 1:
            # カラー原画 → グレイスケール → 白黒 2値画像
            img1 = self.img_binarization(img)
        elif process_mode == 2:
            # 罫線消去
            img1 = self.delete_line(img)
        elif process_mode == 3:
            # 画像の傾き補正
            thr, lln, gap = self.get_autohoughLine(img)
            arg = self.get_degree(img, thr, lln, gap)
            img1 = self.rotate_img(img, arg)
            if self.logf:
                print('\n Rotete degree: ', arg)
        elif process_mode == 4:
            # 印影除去
            mask, img1 = self.detect_color(img, 320, 50, 5, 100, 50, 100, 2)

        elif process_mode == 5:
            # 黒抽出
            mask, img0 = self.detect_color(img, 0, 359, 0, 30, 0, 50, 0)
            img1 = cv2.bitwise_not(mask)    # 反転
        return img1            

##---------------
    ### ハフライン検出のための自動パラメータの計算 (画像サイズの違いによる)
    ###  戻り値: threshhold, minLineLength, maxLineGap
    def get_autohoughLine(self, img, logf = True):
        h, w = img.shape[:2]
        thr = 100
        lln = int(w/18)
        if lln < 200:                   # 2020/02/07
            lln = 200

        gap = int(w/1000) + 4
        if self.logf:
            print('\n Threshhold={}, MinLineLength={}, MaxLineGap={},  width={}, height={}'.format(thr, lln, gap, w, h))
        return thr, lln, gap

    ### 二点間の距離を求める
    ###  戻り値: distance
    def get_distance(self, x1, y1, x2, y2):
        d = math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)
        return d

    ### 画像の傾き検出
    ###  戻り値: degree (水平からの傾き角度/座標系の違いからこの値が補正すべき角度になる)
    def get_degree(self, img, thr, lln, gap):
        l_img = img.copy()
        img0 = img.copy()
    
        gray_image = cv2.cvtColor(l_img, cv2.COLOR_BGR2GRAY)
        edges = cv2.Canny(gray_image,50,150,apertureSize = 3)
        lines = cv2.HoughLinesP(edges, 1, np.pi/180, thr, lln, gap)

        if self.logf:
            print('\n -20° 〜 +20° Horizontal Line')

        sum_arg = 0;
        count = 0;
        for line in lines:
            for x1,y1,x2,y2 in line:
                # 直線を描画する。
                cv2.line(img0, (x1, y1), (x2, y2), (0, 0, 255), 2)

                arg = math.degrees(math.atan2((y2-y1), (x2-x1)))
                HORIZONTAL = 0
                DIFF = 20 # 許容誤差 -> -20 - +20 を本来の水平線と考える
                #arg != 0を条件に追加し、傾きの平均を0に寄りにくくした。
                if arg != 0 and arg > HORIZONTAL - DIFF and arg < HORIZONTAL + DIFF : 
                    if self.logf:
                        print(' count= {}, degree= {}'.format(count, arg))
                    sum_arg += arg;
                    count += 1

        # 処理結果
        if count == 0:
            return HORIZONTAL
        else:
            return (sum_arg / count);

    ### 角度領域の計算
    ###  入力: 0 〜 360
    ###  戻り値: 0 〜 179 OpenCV領域 min,max=180 エラー
    def calc_hue_area(self, hmin, hmax):
        h_min1 = 360
        h_max1 = 360
        h_min2 = 360
        h_max2 = 360
        if hmin >= 0 and hmin <=360 and hmax >= 0 and hmax <= 360:
            if hmin < hmax:
                h_min1 = hmin
                h_max1 = hmax
            else:
                h_min1 = 0
                h_max1 = hmax
                h_min2 = hmin
                h_max2 = 360

        # OpenCV の範囲に変換
        h_min1 = int(h_min1/2)
        h_max1 = int(h_max1/2)
        h_min2 = int(h_min2/2)
        h_max2 = int(h_max2/2)
        if h_max2 > h_min2 and h_max2 ==180:
            h_max2 = 179

        return h_min1, h_max1, h_min2, h_max2

##---------------
# テストルーチン
# キー入力
#   1:      カラー原画 → グレイスケール → 白黒 2値画像
#   2:      罫線消去
#   3:      画像の傾き補正
#   4:      印影除去
#   5:      黒抽出
#   q, Esc: 終了

def main(testmode):
    window_name = 'Input Image'
    window_name1 = 'Processing Image'
    testfile = ['None','pros_test1.png','pros_test3.png', 'tilt_sample.png', 'pros_test4.png', 'pros_test2.png']
    img1 = None
    imgpros = ImagePreprocess(True)                 # 初期化

    img = cv2.imread('images/' + testfile[testmode])
    if img is None:
        print(' Unable to read the input.')
        quit()

    cv2.namedWindow(window_name, flags=cv2.WINDOW_AUTOSIZE | cv2.WINDOW_GUI_NORMAL) 
    cv2.imshow(window_name, img)
    cv2.moveWindow(window_name, 100, 0)

    img1 = imgpros.image_processing_execution(img, testmode)
    if img1 is not None:
        cv2.namedWindow(window_name1, flags=cv2.WINDOW_AUTOSIZE | cv2.WINDOW_GUI_NORMAL) 
        cv2.imshow(window_name1, img1)
        cv2.moveWindow(window_name1, 100 + img.shape[1], 0)

    while(True):
        key = cv2.waitKey(1)
        if key == 27 or key == 113:                 # 'esc' or 'q'
            break
        elif key >= ord('1') and key <= ord('5'):   # テストモード変更
            key = key - ord('0')
            break

    cv2.destroyAllWindows()
    return key

if __name__ == "__main__":
    loopf = True
    mode = 1
    
    while loopf:
        mode = main(mode)
        if mode == 27 or mode == 113:
            loopf = False

