# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library YAML file process
##
##               2022.01.11 Masahiro Izutsu
##------------------------------------------
## mylib_yaml.py
## 2022.01.14 追加書き込み対応

import yaml
import codecs
import os

MODULE_NAME = 'YamlProcess'

class YamlProcess:
    logf = False
    out_path = None
    
    # 初期化
    #   out_path:   出力ファイル・パス
    #   flg:        ログ出力フラグ
    def __init__(self, out_path, flg = False):
        self.logf = flg
        self.out_path = out_path

    # セクション/キーの追加書き込み
    #   section:    セクション名
    #   key:        データ名
    #   data:       オブジェクト
    def write_append_key(self, section, key, data):
        if os.path.isfile(self.out_path):
            with codecs.open(self.out_path, 'r+', 'utf-8') as f:
                obj = None
                try:
                    obj = yaml.safe_load(f)
                except yaml.YAMLError as exc:
                    print(exc)

                if obj != None and section in obj:
                    obj[section][key] = data
                    f.seek(0)
                    yaml.dump(obj, f, encoding='utf-8', allow_unicode=True)                
                    if self.logf:
                        print(yaml.dump(obj))
                else:
                    new_data = {key: data}
                    self.write_append(section, new_data)
        else:
            new_data = {key: data}
            self.write_section(section, new_data)

    # セクションの追加書き込み
    #   section:    セクション名
    #   data:       オブジェクト
    def write_append(self, section, data):
        if os.path.isfile(self.out_path):
            with codecs.open(self.out_path, 'r+', 'utf-8') as f:
                try:
                    obj = yaml.safe_load(f)
                    obj[section] = data
                    f.seek(0)
                    yaml.dump(obj, f, encoding='utf-8', allow_unicode=True)                
                    if self.logf:
                        print(yaml.dump(obj))

                except yaml.YAMLError as exc:
                    print('[{}:write_append()] yaml.YAMLError !!  '.format(MODULE_NAME), exc)

        else:
            self.write_section(section, data)

    # セクションの書き込み
    #   section:    セクション名
    #   data:       オブジェクト
    def write_section(self, section, data):
        obj = {section: data}
        self.write_object(obj)

    # オブジェクトの書き込み
    #   obj:       オブジェクト
    def write_object(self, obj):
        with codecs.open(self.out_path, 'w', 'utf-8') as f:
            yaml.dump(obj, f, encoding='utf-8', allow_unicode=True)

        if self.logf:
            print(yaml.dump(obj))

    # セクション/サブ・セクション/キーの読み出し
    #   section:    セクション名
    #   sub_sec:    サブ・セクション名
    #   key:        データ名
    def load_section_sub_key(self, section, sub_sec, key):
        obj = self.load_section_key(section, sub_sec)
        data = None
        if obj != None and key in obj:
            data = obj[key]
        return data

    # セクション/キーの読み出し
    #   section:    セクション名
    #   key:        データ名
    def load_section_key(self, section, key):
        obj = self.load_section(section)
        data = None
        if obj != None and key in obj:
            data = obj[key]
        return data

    # セクションの読み出し
    #   戻り値:      セクション・オブジェクト
    def load_section(self, section):
        obj = self.load_object()
        data = None
        if obj != None and section in obj:
            data = obj[section]
        return data
        
    # オブジェクトの読み出し
    #   戻り値:      オブジェクト
    def load_object(self):
        obj = None
        with codecs.open(self.out_path, 'r', 'utf-8') as f:
            try:
                obj = yaml.safe_load(f)
            except yaml.YAMLError as exc:
                print(exc)
        return obj


##---------------
def main():
    from os.path import expanduser

    # 配置ファイル作成
    out_path1 = expanduser('./output_test.yaml')
    name1 = 'file1'
    myyaml1 = YamlProcess(out_path1, True)

    path = 'C:/filename'
    base_cord = [640, 480]
    locat = [10, 20, 30, 40]
    prepros = 0

    # 基本情報を書く
    data = {'.FileName': path, '.BasicCoordinates': base_cord}
    myyaml1.write_append(name1, data)

    # 配置情報を書く
    data = {'Location': locat, 'PreProcess': prepros, 'Text': '請求書'}
    myyaml1.write_append_key(name1, 'KyeName', data)
    data = {'Location': locat, 'PreProcess': prepros, 'Text': '2022年01月11日'}
    myyaml1.write_append_key(name1, 'Date', data)
    data = {'Location': locat, 'PreProcess': prepros, 'Text': '会社名'}
    myyaml1.write_append_key(name1, 'CompName', data)
    data = {'Location': locat, 'PreProcess': prepros, 'Text': '案件名'}
    myyaml1.write_append_key(name1, 'Title', data)
    data = {'Location': locat, 'PreProcess': prepros, 'Text': '税抜金額'}
    myyaml1.write_append_key(name1, 'B4Tax', data)
    data = {'Location': locat, 'PreProcess': prepros, 'Text': '税込み金額'}
    myyaml1.write_append_key(name1, 'TotalMoney', data)

    # キーテーブル作成
    out_path2 = expanduser('./output_table.yaml')
    myyaml2 = YamlProcess(out_path2, True)

    key_table = ['KyeName', 'Date', 'CompName', 'Title', 'B4Tax', 'TotalMoney']
    myyaml2.write_append('KeyTable', key_table)

    #----------------------------------
    
    # キーテーブルを得る
    table = myyaml2.load_section('KeyTable')
    print(table, len(table))

    data = myyaml1.load_section(name1)

    # 基本情報を得る
    base_size = data['.BasicCoordinates']
    file_name = data['.FileName']
    print(base_size, file_name)
    
    # 配置情報を得る
    location = []
    prepros = []
    text = []
    i = 0
    for tn in table:
        location.append(data[tn]['Location'])
        prepros.append(data[tn]['PreProcess'])
        text.append(data[tn]['Text'])
        print(location[i], prepros[i], text[i])
        i = i + 1

if __name__ == "__main__":
    main()

